@@ -13,8 +13,8 @@
 # under the License.
 
 from __future__ import annotations
-
-from typing import TYPE_CHECKING, Any, Dict
+from typing_extensions import Literal
+from typing import TYPE_CHECKING, Any, Dict, Optional
 
 from supertokens_python.ingredients.emaildelivery.types import (
     EmailDeliveryConfig,
@@ -30,21 +30,8 @@ if TYPE_CHECKING:
 
     from supertokens_python.supertokens import AppInfo
 
-    from .interfaces import APIInterface, RecipeInterface
-    from .types import User, VerificationEmailTemplateVars
-
-
-def default_get_email_verification_url(
-    app_info: AppInfo,
-) -> Callable[[User, Dict[str, Any]], Awaitable[str]]:
-    async def func(_: User, __: Dict[str, Any]):
-        return (
-            app_info.website_domain.get_as_string_dangerous()
-            + app_info.website_base_path.get_as_string_dangerous()
-            + "/verify-email"
-        )
-
-    return func
+    from .interfaces import APIInterface, RecipeInterface, TypeGetEmailForUserIdFunction
+    from .types import User, VerificationEmailTemplateVars, EmailTemplateVars
 
 
 class OverrideConfig:
@@ -57,86 +44,60 @@ class OverrideConfig:
         self.apis = apis
 
 
-class ParentRecipeEmailVerificationConfig:
-    def __init__(
-        self,
-        get_email_for_user_id: Callable[[str, Dict[str, Any]], Awaitable[str]],
-        override: Union[OverrideConfig, None] = None,
-        get_email_verification_url: Union[
-            Callable[[User, Dict[str, Any]], Awaitable[str]], None
-        ] = None,
-        create_and_send_custom_email: Union[
-            Callable[[User, str, Dict[str, Any]], Awaitable[None]], None
-        ] = None,
-        email_delivery: Union[
-            EmailDeliveryConfig[VerificationEmailTemplateVars], None
-        ] = None,
-    ):
-        self.override = override
-        self.get_email_verification_url = get_email_verification_url
-        self.get_email_for_user_id = get_email_for_user_id
-        self.email_delivery = email_delivery
-        self.create_and_send_custom_email = create_and_send_custom_email
-
-        if create_and_send_custom_email:
-            # Note: This will appear twice because `InputEmailVerificationConfig` will also produce same warning.
-            deprecated_warn(
-                "create_and_send_custom_email is deprecated. Please use email delivery config instead"
-            )
+MODE_TYPE = Literal["REQUIRED", "OPTIONAL"]
 
 
 class EmailVerificationConfig:
     def __init__(
         self,
-        override: OverrideConfig,
-        get_email_verification_url: Callable[[User, Dict[str, Any]], Awaitable[str]],
-        get_email_for_user_id: Callable[[str, Dict[str, Any]], Awaitable[str]],
+        mode: MODE_TYPE,
         get_email_delivery_config: Callable[
             [], EmailDeliveryConfigWithService[VerificationEmailTemplateVars]
         ],
+        get_email_for_user_id: Optional[TypeGetEmailForUserIdFunction],
+        override: OverrideConfig,
     ):
-        self.get_email_for_user_id = get_email_for_user_id
-        self.get_email_verification_url = get_email_verification_url
+        self.mode = mode
         self.override = override
         self.get_email_delivery_config = get_email_delivery_config
+        self.get_email_for_user_id = get_email_for_user_id
 
 
 def validate_and_normalise_user_input(
-    app_info: AppInfo, config: ParentRecipeEmailVerificationConfig
+    app_info: AppInfo,
+    mode: MODE_TYPE,
+    email_delivery: Union[EmailDeliveryConfig[EmailTemplateVars], None] = None,
+    get_email_for_user_id: Optional[TypeGetEmailForUserIdFunction] = None,
+    create_and_send_custom_email: Union[
+        Callable[[User, str, Dict[str, Any]], Awaitable[None]], None
+    ] = None,
+    override: Union[OverrideConfig, None] = None,
 ) -> EmailVerificationConfig:
-    if not isinstance(config, ParentRecipeEmailVerificationConfig):  # type: ignore
-        raise ValueError(
-            "config must be an instance of ParentRecipeEmailVerificationConfig"
+    if create_and_send_custom_email:
+        deprecated_warn(
+            "create_and_send_custom_email is deprecated. Please use email delivery config instead"
         )
 
-    get_email_verification_url = (
-        config.get_email_verification_url
-        if config.get_email_verification_url is not None
-        else default_get_email_verification_url(app_info)
-    )
+    if mode not in ["REQUIRED", "OPTIONAL"]:
+        raise ValueError(
+            "Email Verification recipe mode must be one of 'REQUIRED' or 'OPTIONAL'"
+        )
 
     def get_email_delivery_config() -> EmailDeliveryConfigWithService[
         VerificationEmailTemplateVars
     ]:
-        email_service = (
-            config.email_delivery.service if config.email_delivery is not None else None
-        )
+        email_service = email_delivery.service if email_delivery is not None else None
         if email_service is None:
             email_service = BackwardCompatibilityService(
-                app_info, config.create_and_send_custom_email
+                app_info, create_and_send_custom_email
             )
 
-        if (
-            config.email_delivery is not None
-            and config.email_delivery.override is not None
-        ):
-            override = config.email_delivery.override
+        if email_delivery is not None and email_delivery.override is not None:
+            override = email_delivery.override
         else:
             override = None
         return EmailDeliveryConfigWithService(email_service, override=override)
 
-    override = config.override
-
     if override is not None and not isinstance(override, OverrideConfig):  # type: ignore
         raise ValueError("override must be of type OverrideConfig or None")
 
@@ -144,8 +105,8 @@ def validate_and_normalise_user_input(
         override = OverrideConfig()
 
     return EmailVerificationConfig(
-        override,
-        get_email_verification_url,
-        config.get_email_for_user_id,
+        mode,
         get_email_delivery_config,
+        get_email_for_user_id,
+        override,
     )
