@@ -29,7 +29,14 @@ async def handle_signout_api(api_implementation: APIInterface, api_options: APIO
         return None
     user_context = default_user_context(api_options.request)
 
-    response = await api_implementation.signout_post(api_options, user_context)
+    session = await api_options.recipe_implementation.get_session(
+        request=api_options.request,
+        anti_csrf_check=None,
+        session_required=False,
+        user_context=user_context,
+    )
+
+    response = await api_implementation.signout_post(session, api_options, user_context)
     if api_options.response is None:
         raise Exception("Should never come here")
     return send_200_response(response.to_json(), api_options.response)
