@@ -11,14 +11,14 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-SESSION_REFRESH = '/session/refresh'
-SIGNOUT = '/signout'
-ACCESS_TOKEN_COOKIE_KEY = 'sAccessToken'
-REFRESH_TOKEN_COOKIE_KEY = 'sRefreshToken'
-ID_REFRESH_TOKEN_COOKIE_KEY = 'sIdRefreshToken'
-FRONT_TOKEN_HEADER_SET_KEY = 'front-token'
-ANTI_CSRF_HEADER_KEY = 'anti-csrf'
-RID_HEADER_KEY = 'rid'
-ID_REFRESH_TOKEN_HEADER_SET_KEY = 'id-refresh-token'
-ID_REFRESH_TOKEN_HEADER_GET_KEY = 'id-refresh-token'
-ACCESS_CONTROL_EXPOSE_HEADERS = 'Access-Control-Expose-Headers'
+SESSION_REFRESH = "/session/refresh"
+SIGNOUT = "/signout"
+ACCESS_TOKEN_COOKIE_KEY = "sAccessToken"
+REFRESH_TOKEN_COOKIE_KEY = "sRefreshToken"
+ID_REFRESH_TOKEN_COOKIE_KEY = "sIdRefreshToken"
+FRONT_TOKEN_HEADER_SET_KEY = "front-token"
+ANTI_CSRF_HEADER_KEY = "anti-csrf"
+RID_HEADER_KEY = "rid"
+ID_REFRESH_TOKEN_HEADER_SET_KEY = "id-refresh-token"
+ID_REFRESH_TOKEN_HEADER_GET_KEY = "id-refresh-token"
+ACCESS_CONTROL_EXPOSE_HEADERS = "Access-Control-Expose-Headers"
