@@ -27,19 +27,31 @@ from fastapi.testclient import TestClient
 from pytest import fixture, mark
 from supertokens_python import InputAppInfo, SupertokensConfig, init
 from supertokens_python.framework.fastapi import get_middleware
-from supertokens_python.ingredients.smsdelivery.services.supertokens import \
-    SUPERTOKENS_SMS_SERVICE_URL
+from supertokens_python.ingredients.smsdelivery.services.supertokens import (
+    SUPERTOKENS_SMS_SERVICE_URL,
+)
 from supertokens_python.ingredients.smsdelivery.types import (
-    SMSDeliveryConfig, SMSDeliveryInterface, TwilioSettings, SMSContent, TwilioServiceInterface)
+    SMSDeliveryConfig,
+    SMSDeliveryInterface,
+    TwilioSettings,
+    SMSContent,
+    TwilioServiceInterface,
+)
 from supertokens_python.querier import Querier
 from supertokens_python.recipe import passwordless, session
-from supertokens_python.recipe.passwordless.smsdelivery.services.twilio import \
-    TwilioService
-from supertokens_python.recipe.passwordless.types import \
-    SMSTemplateVars
+from supertokens_python.recipe.passwordless.smsdelivery.services.twilio import (
+    TwilioService,
+)
+from supertokens_python.recipe.passwordless.types import SMSTemplateVars
 from supertokens_python.utils import is_version_gte
-from tests.utils import (clean_st, reset, setup_st, sign_in_up_request_phone,
-                         start_st)
+from tests.utils import (
+    clean_st,
+    reset,
+    setup_st,
+    sign_in_up_request_phone,
+    start_st,
+    sign_in_up_request_code_resend,
+)
 
 respx_mock = respx.MockRouter
 
@@ -55,22 +67,24 @@ def teardown_function(_):
     clean_st()
 
 
-@fixture(scope='function')
+@fixture(scope="function")
 async def driver_config_client():
     app = FastAPI()
     app.add_middleware(get_middleware())
 
-    @app.get('/login')
+    @app.get("/login")
     async def login(_request: Request):  # type: ignore
-        user_id = 'userId'
+        user_id = "userId"
         # await create_new_session(request, user_id, {}, {})
-        return {'userId': user_id}
+        return {"userId": user_id}
 
     return TestClient(app)
 
 
 @mark.asyncio
-async def test_pless_login_default_backward_compatibility(driver_config_client: TestClient):
+async def test_pless_login_default_backward_compatibility(
+    driver_config_client: TestClient,
+):
     "Passwordless login: test default backward compatibility api being called"
     app_name = ""
     phone_number = ""
@@ -78,20 +92,24 @@ async def test_pless_login_default_backward_compatibility(driver_config_client:
     url_with_link_code = ""
     user_input_code = ""
     api_key = ""
+    resend_called = False
 
     init(
-        supertokens_config=SupertokensConfig('http://localhost:3567'),
+        supertokens_config=SupertokensConfig("http://localhost:3567"),
         app_info=InputAppInfo(
             app_name="ST",
             api_domain="http://api.supertokens.io",
             website_domain="http://supertokens.io",
-            api_base_path="/auth"
+            api_base_path="/auth",
         ),
-        framework='fastapi',
-        recipe_list=[passwordless.init(
-            contact_config=passwordless.ContactPhoneOnlyConfig(),
-            flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
-        ), session.init()]
+        framework="fastapi",
+        recipe_list=[
+            passwordless.init(
+                contact_config=passwordless.ContactPhoneOnlyConfig(),
+                flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
+            ),
+            session.init(),
+        ],
     )
     start_st()
 
@@ -116,21 +134,55 @@ async def test_pless_login_default_backward_compatibility(driver_config_client:
 
     with respx_mock(assert_all_mocked=False) as mocker:
         mocker.route(host="localhost").pass_through()
-        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(side_effect=api_side_effect)
+        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(
+            side_effect=api_side_effect
+        )
         resp = sign_in_up_request_phone(driver_config_client, "+919909909998", True)
+        body = resp.json()
+
+        assert resp.status_code == 200
+        assert mocked_route.called
+
+    def code_resend_api_side_effect(request: httpx.Request):
+        nonlocal app_name, phone_number, code_lifetime, url_with_link_code, user_input_code, resend_called
+        body = json.loads(request.content)["smsInput"]
+
+        assert (
+            body["userInputCode"] != user_input_code
+        )  # Default resend generates a new code
+
+        app_name = body["appName"]
+        phone_number = body["phoneNumber"]
+        code_lifetime = body["codeLifetime"]
+        url_with_link_code = body["urlWithLinkCode"]
+        user_input_code = body["userInputCode"]
+        resend_called = True
+
+        return httpx.Response(200, json={})
+
+    with respx_mock(assert_all_mocked=False) as mocker:
+        mocker.route(host="localhost").pass_through()
+        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(
+            side_effect=code_resend_api_side_effect
+        )
+        resp = sign_in_up_request_code_resend(
+            driver_config_client, body["deviceId"], body["preAuthSessionId"], True
+        )
 
         assert resp.status_code == 200
         assert mocked_route.called
 
         assert app_name == "ST"
         assert phone_number == "+919909909998"
-        assert all([url_with_link_code, user_input_code, code_lifetime])
+        assert all([url_with_link_code, user_input_code, code_lifetime, resend_called])
         assert code_lifetime > 0
         assert api_key is None
 
 
 @mark.asyncio
-async def test_pless_login_default_backward_compatibility_no_suppress_error_for_4xx_5xx(driver_config_client: TestClient):
+async def test_pless_login_default_backward_compatibility_no_suppress_error_for_4xx_5xx(
+    driver_config_client: TestClient,
+):
     "Passwordless login: test default backward compatibility api being called, error message sent back to user if core sends 400-599 (except 429)"
     app_name = ""
     phone = ""
@@ -139,18 +191,21 @@ async def test_pless_login_default_backward_compatibility_no_suppress_error_for_
     user_input_code = ""
 
     init(
-        supertokens_config=SupertokensConfig('http://localhost:3567'),
+        supertokens_config=SupertokensConfig("http://localhost:3567"),
         app_info=InputAppInfo(
             app_name="ST",
             api_domain="http://api.supertokens.io",
             website_domain="http://supertokens.io",
-            api_base_path="/auth"
+            api_base_path="/auth",
         ),
-        framework='fastapi',
-        recipe_list=[passwordless.init(
-            contact_config=passwordless.ContactPhoneOnlyConfig(),
-            flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
-        ), session.init()]
+        framework="fastapi",
+        recipe_list=[
+            passwordless.init(
+                contact_config=passwordless.ContactPhoneOnlyConfig(),
+                flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
+            ),
+            session.init(),
+        ],
     )
     start_st()
 
@@ -173,7 +228,9 @@ async def test_pless_login_default_backward_compatibility_no_suppress_error_for_
 
     with respx_mock(assert_all_mocked=False) as mocker:
         mocker.route(host="localhost").pass_through()
-        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(side_effect=api_side_effect)
+        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(
+            side_effect=api_side_effect
+        )
         try:
             sign_in_up_request_phone(driver_config_client, "+919909909998", True)
         except Exception as e:
@@ -187,7 +244,9 @@ async def test_pless_login_default_backward_compatibility_no_suppress_error_for_
 
 
 @mark.asyncio
-async def test_pless_login_default_backward_compatibility_suppress_error_for_429(driver_config_client: TestClient):
+async def test_pless_login_default_backward_compatibility_suppress_error_for_429(
+    driver_config_client: TestClient,
+):
     "Passwordless login: test default backward compatibility api being called, error message not sent back to user if core sends 429 (Too many requests from client) but prints quota reached."
     app_name = ""
     phone = ""
@@ -196,18 +255,21 @@ async def test_pless_login_default_backward_compatibility_suppress_error_for_429
     user_input_code = ""
 
     init(
-        supertokens_config=SupertokensConfig('http://localhost:3567'),
+        supertokens_config=SupertokensConfig("http://localhost:3567"),
         app_info=InputAppInfo(
             app_name="ST",
             api_domain="http://api.supertokens.io",
             website_domain="http://supertokens.io",
-            api_base_path="/auth"
+            api_base_path="/auth",
         ),
-        framework='fastapi',
-        recipe_list=[passwordless.init(
-            contact_config=passwordless.ContactPhoneOnlyConfig(),
-            flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
-        ), session.init()]
+        framework="fastapi",
+        recipe_list=[
+            passwordless.init(
+                contact_config=passwordless.ContactPhoneOnlyConfig(),
+                flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
+            ),
+            session.init(),
+        ],
     )
     start_st()
 
@@ -232,7 +294,9 @@ async def test_pless_login_default_backward_compatibility_suppress_error_for_429
     with respx_mock(assert_all_mocked=False) as mocker:
         with redirect_stdout(f):
             mocker.route(host="localhost").pass_through()
-            mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(side_effect=api_side_effect)
+            mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(
+                side_effect=api_side_effect
+            )
             resp = sign_in_up_request_phone(driver_config_client, "+919909909998", True)
 
             assert resp.status_code == 200
@@ -256,7 +320,9 @@ async def test_pless_login_backward_compatibility(driver_config_client: TestClie
     url_with_link_code = ""
     user_input_code = ""
 
-    async def create_and_send_custom_text_message(input_: SMSTemplateVars, _: Dict[str, Any]):
+    async def create_and_send_custom_text_message(
+        input_: SMSTemplateVars, _: Dict[str, Any]
+    ):
         nonlocal phone, code_lifetime, url_with_link_code, user_input_code
         phone = input_.phone_number
         code_lifetime = input_.code_life_time
@@ -264,20 +330,23 @@ async def test_pless_login_backward_compatibility(driver_config_client: TestClie
         user_input_code = input_.user_input_code
 
     init(
-        supertokens_config=SupertokensConfig('http://localhost:3567'),
+        supertokens_config=SupertokensConfig("http://localhost:3567"),
         app_info=InputAppInfo(
             app_name="ST",
             api_domain="http://api.supertokens.io",
             website_domain="http://supertokens.io",
-            api_base_path="/auth"
+            api_base_path="/auth",
         ),
-        framework='fastapi',
-        recipe_list=[passwordless.init(
-            contact_config=passwordless.ContactPhoneOnlyConfig(
-                create_and_send_custom_text_message=create_and_send_custom_text_message,
+        framework="fastapi",
+        recipe_list=[
+            passwordless.init(
+                contact_config=passwordless.ContactPhoneOnlyConfig(
+                    create_and_send_custom_text_message=create_and_send_custom_text_message,
+                ),
+                flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
             ),
-            flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
-        ), session.init()]
+            session.init(),
+        ],
     )
     start_st()
 
@@ -286,6 +355,23 @@ async def test_pless_login_backward_compatibility(driver_config_client: TestClie
         return
 
     resp = sign_in_up_request_phone(driver_config_client, "+919909909998", True)
+    body = resp.json()
+
+    assert resp.status_code == 200
+
+    assert phone == "+919909909998"
+    assert all([url_with_link_code, user_input_code, code_lifetime])
+    assert code_lifetime > 0
+
+    # Resend
+    phone = ""
+    code_lifetime = 0
+    url_with_link_code = ""
+    user_input_code = ""
+
+    resp = sign_in_up_request_code_resend(
+        driver_config_client, body["deviceId"], body["preAuthSessionId"]
+    )
 
     assert resp.status_code == 200
 
@@ -302,11 +388,14 @@ async def test_pless_login_custom_override(driver_config_client: TestClient):
     url_with_link_code = ""
     user_input_code = ""
     app_name = ""
+    resend_called = False
 
     def sms_delivery_override(oi: SMSDeliveryInterface[SMSTemplateVars]):
         oi_send_sms = oi.send_sms
 
-        async def send_sms(template_vars: SMSTemplateVars, user_context: Dict[str, Any]):
+        async def send_sms(
+            template_vars: SMSTemplateVars, user_context: Dict[str, Any]
+        ):
             nonlocal phone, url_with_link_code, user_input_code, code_lifetime
             phone = template_vars.phone_number
             url_with_link_code = template_vars.url_with_link_code
@@ -319,22 +408,25 @@ async def test_pless_login_custom_override(driver_config_client: TestClient):
         return oi
 
     init(
-        supertokens_config=SupertokensConfig('http://localhost:3567'),
+        supertokens_config=SupertokensConfig("http://localhost:3567"),
         app_info=InputAppInfo(
             app_name="ST",
             api_domain="http://api.supertokens.io",
             website_domain="http://supertokens.io",
-            api_base_path="/auth"
+            api_base_path="/auth",
         ),
-        framework='fastapi',
-        recipe_list=[passwordless.init(
-            contact_config=passwordless.ContactPhoneOnlyConfig(),
-            flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
-            sms_delivery=SMSDeliveryConfig(
-                service=None,
-                override=sms_delivery_override,
-            )
-        ), session.init()]
+        framework="fastapi",
+        recipe_list=[
+            passwordless.init(
+                contact_config=passwordless.ContactPhoneOnlyConfig(),
+                flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
+                sms_delivery=SMSDeliveryConfig(
+                    service=None,
+                    override=sms_delivery_override,
+                ),
+            ),
+            session.init(),
+        ],
     )
     start_st()
 
@@ -351,8 +443,36 @@ async def test_pless_login_custom_override(driver_config_client: TestClient):
 
     with respx_mock(assert_all_mocked=False) as mocker:
         mocker.route(host="localhost").pass_through()
-        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(side_effect=api_side_effect)
+        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(
+            side_effect=api_side_effect
+        )
         resp = sign_in_up_request_phone(driver_config_client, "+919909909998", True)
+        body = resp.json()
+
+        assert resp.status_code == 200
+        assert mocked_route.called
+
+    def code_resend_api_side_effect(request: httpx.Request):
+        nonlocal app_name, phone, code_lifetime, url_with_link_code, user_input_code, resend_called
+        body = json.loads(request.content)["smsInput"]
+
+        app_name = body["appName"]
+        phone = body["phoneNumber"]
+        code_lifetime = body["codeLifetime"]
+        url_with_link_code = body["urlWithLinkCode"]
+        user_input_code = body["userInputCode"]
+        resend_called = True
+
+        return httpx.Response(200, json={})
+
+    with respx_mock(assert_all_mocked=False) as mocker:
+        mocker.route(host="localhost").pass_through()
+        mocked_route = mocker.post(SUPERTOKENS_SMS_SERVICE_URL).mock(
+            side_effect=code_resend_api_side_effect
+        )
+        resp = sign_in_up_request_code_resend(
+            driver_config_client, body["deviceId"], body["preAuthSessionId"], True
+        )
 
         assert resp.status_code == 200
         assert mocked_route.called
@@ -364,12 +484,16 @@ async def test_pless_login_custom_override(driver_config_client: TestClient):
 
 
 @mark.asyncio
-async def test_pless_login_smtp_service(driver_config_client: TestClient):
-    "Passwordless login: test smtp service"
+async def test_pless_login_twilio_service(driver_config_client: TestClient):
+    "Passwordless login: test twilio service"
     phone = ""
     code_lifetime = 0
     user_input_code = ""
-    get_content_called, send_raw_email_called, outer_override_called = False, False, False
+    get_content_called, send_raw_email_called, outer_override_called = (
+        False,
+        False,
+        False,
+    )
     twilio_api_called = False
 
     def twilio_service_override(oi: TwilioServiceInterface[SMSTemplateVars]):
@@ -391,17 +515,16 @@ async def test_pless_login_smtp_service(driver_config_client: TestClient):
 
             await oi_send_raw_sms(content, _user_context, from_, messaging_service_sid)
 
-        async def get_content_override(template_vars: SMSTemplateVars, _user_context: Dict[str, Any]) -> SMSContent:
+        async def get_content_override(
+            template_vars: SMSTemplateVars, _user_context: Dict[str, Any]
+        ) -> SMSContent:
             nonlocal get_content_called, user_input_code, code_lifetime
             get_content_called = True
 
             user_input_code = template_vars.user_input_code or ""
             code_lifetime = template_vars.code_life_time
 
-            return SMSContent(
-                body=user_input_code,
-                to_phone=template_vars.phone_number
-            )
+            return SMSContent(body=user_input_code, to_phone=template_vars.phone_number)
 
         oi.send_raw_sms = send_raw_email_override
         oi.get_content = get_content_override
@@ -417,10 +540,14 @@ async def test_pless_login_smtp_service(driver_config_client: TestClient):
         override=twilio_service_override,
     )
 
-    def sms_delivery_override(oi: SMSDeliveryInterface[SMSTemplateVars]) -> SMSDeliveryInterface[SMSTemplateVars]:
+    def sms_delivery_override(
+        oi: SMSDeliveryInterface[SMSTemplateVars],
+    ) -> SMSDeliveryInterface[SMSTemplateVars]:
         oi_send_sms = oi.send_sms
 
-        async def send_sms_override(template_vars: SMSTemplateVars, user_context: Dict[str, Any]):
+        async def send_sms_override(
+            template_vars: SMSTemplateVars, user_context: Dict[str, Any]
+        ):
             nonlocal outer_override_called
             outer_override_called = True
             await oi_send_sms(template_vars, user_context)
@@ -429,22 +556,25 @@ async def test_pless_login_smtp_service(driver_config_client: TestClient):
         return oi
 
     init(
-        supertokens_config=SupertokensConfig('http://localhost:3567'),
+        supertokens_config=SupertokensConfig("http://localhost:3567"),
         app_info=InputAppInfo(
             app_name="ST",
             api_domain="http://api.supertokens.io",
             website_domain="http://supertokens.io",
-            api_base_path="/auth"
+            api_base_path="/auth",
         ),
-        framework='fastapi',
-        recipe_list=[passwordless.init(
-            contact_config=passwordless.ContactPhoneOnlyConfig(),
-            flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
-            sms_delivery=SMSDeliveryConfig(
-                service=twilio_sms_delivery_service,
-                override=sms_delivery_override,
-            )
-        ), session.init()]
+        framework="fastapi",
+        recipe_list=[
+            passwordless.init(
+                contact_config=passwordless.ContactPhoneOnlyConfig(),
+                flow_type="USER_INPUT_CODE_AND_MAGIC_LINK",
+                sms_delivery=SMSDeliveryConfig(
+                    service=twilio_sms_delivery_service,
+                    override=sms_delivery_override,
+                ),
+            ),
+            session.init(),
+        ],
     )
     start_st()
 
@@ -460,9 +590,42 @@ async def test_pless_login_smtp_service(driver_config_client: TestClient):
 
     m: requests_mock.Mocker
     with requests_mock.Mocker(real_http=True) as m:
-        m.post("https://api.twilio.com/2010-04-01/Accounts/ACTWILIO_ACCOUNT_SID/Messages.json", json=json_callback)
+        m.post(
+            "https://api.twilio.com/2010-04-01/Accounts/ACTWILIO_ACCOUNT_SID/Messages.json",
+            json=json_callback,
+        )
 
         resp = sign_in_up_request_phone(driver_config_client, "+919909909998", True)
+        body = resp.json()
+
+        assert resp.status_code == 200
+
+        assert phone == "+919909909998"
+        assert all([outer_override_called, get_content_called, send_raw_email_called])
+        assert code_lifetime > 0
+        assert twilio_api_called
+
+    # Resend:
+    phone = ""
+    code_lifetime = 0
+    user_input_code = ""
+    get_content_called, send_raw_email_called, outer_override_called = (
+        False,
+        False,
+        False,
+    )
+    twilio_api_called = False
+
+    m: requests_mock.Mocker
+    with requests_mock.Mocker(real_http=True) as m:
+        m.post(
+            "https://api.twilio.com/2010-04-01/Accounts/ACTWILIO_ACCOUNT_SID/Messages.json",
+            json=json_callback,
+        )
+
+        resp = sign_in_up_request_code_resend(
+            driver_config_client, body["deviceId"], body["preAuthSessionId"], True
+        )
 
         assert resp.status_code == 200
 
